@@ -5,7 +5,7 @@
 #include "mailfront.h"
 #include "mailrules.h"
 #include "smtp.h"
-#include "sasl-auth.h"
+#include <cvm/sasl.h>
 
 #include <iobuf/iobuf.h>
 #include <msg/msg.h>
@@ -13,6 +13,8 @@
 str line = {0,0,0};
 str domain_name = {0,0,0};
 
+struct sasl_auth saslauth = { .prefix = "334 " };
+
 extern unsigned long maxnotimpl;
 
 int smtp_mainloop(void)
@@ -37,7 +39,8 @@ int smtp_mainloop(void)
 
   if ((resp = handle_init()) != 0) { respond_resp(resp, 1); return 1; }
 
-  if (!sasl_auth_init()) return respond(421, 1, "Failed to initialize AUTH");
+  if (!sasl_auth_init(&saslauth))
+    return respond(421, 1, "Failed to initialize AUTH");
 
   if (!respond(220, 1, str_welcome.s)) return 1;
   while (smtp_get_line())
