@@ -6,6 +6,8 @@ mkdir -p $tmp
 PATH="$src:/bin:/usr/bin:/usr/local/bin"
 tests_failed=0
 tests_count=0
+_UID=`id -u`
+_GID=`id -g`
 sfqmail() { $src/smtpfront-qmail 2>/dev/null | tail -n +2; }
 sfecho() { $src/smtpfront-echo 2>/dev/null | tail -n +2; }
 pfauth() { $src/pop3front-auth "$@" echo Yes. 2>/dev/null | tail -n +2; }
@@ -14,7 +16,9 @@ pfmaildir() { $src/pop3front-maildir "$@" 2>/dev/null | tail -n +2; }
 maildir=$tmp/Maildir
 maildir() {
   rm -rf $maildir
-  mkdir -p $maildir/{cur,new,tmp}
+  mkdir -p $maildir/cur
+  mkdir -p $maildir/new
+  mkdir -p $maildir/tmp
 }
 tstmsg() {
   fn=$1
@@ -30,35 +34,28 @@ TESTLOCALIP=1.2.3.4
 TESTREMOTEIP=5.6.7.8
 TESTLOCALHOST=local.host
 TESTREMOTEHOST=remote.host
+CVM_PWFILE_PATH=$tmp/pwfile
 
 export PROTO TESTLOCALIP TESTREMOTEIP TESTLOCALHOST TESTREMOTEHOST
+export CVM_PWFILE_PATH MAILRULES DATABYTES MAXHOPS PATTERNS MAXNOTIMPL
 
-cat >$tmp/cvm <<EOF
-#!/bin/sh
-req="\`cat -v\`"
-if [ "x\$req" = 'x^Atestuser^@^@testpass^@^@' ]; then
-  printf '\\0\\0010500\\0\\1testuser\\0\\0002500\\0\\0003500\\0\\4Test User\\0\\5$tmp\\0\\6/bin/false\\0\\7testgroup\\0\\0'
-  exit 0
-fi
-printf \\\\0144
-exit 100
-EOF
-chmod +x $tmp/cvm
+echo testuser:testpass:$_UID:$_GID:Test User:$tmp:/bin/false >$CVM_PWFILE_PATH
+ln -s `which cvm-pwfile` $tmp/cvm
 
 run_compare_test() {
   local name=$1
   shift
-  sed -e "s:@SOURCE@:$src:g" >$tmp/expected
-  ( runtest "$@" ) 2>&1 >$tmp/actual-raw
+  sed -e "s:@SOURCE@:$src:g"   	-e "s:@TMPDIR@:$tmp:g"   	-e "s:@UID@:$_UID:" 	-e "s:@GID@:$_GID:" 	>$tmp/expected
+  ( runtest "$@" 2>&1 ) 2>&1 >$tmp/actual-raw
   cat -v $tmp/actual-raw >$tmp/actual
   if ! cmp $tmp/expected $tmp/actual >/dev/null 2>&1
   then
     echo "Test $name $@ failed:"
-	( cd $tmp; diff -u9999 expected actual | tail -n +3; echo; )
-	let tests_failed=tests_failed+1
+	( cd $tmp; diff -U 9999 expected actual | tail -n +3; echo; )
+	tests_failed=$(($tests_failed+1))
   fi
   rm -f $tmp/expected $tmp/actual
-  let tests_count=tests_count+1
+  tests_count=$(($tests_count+1))
 }
 
 ##### Test tests/rules-header-add #####
@@ -68,13 +65,16 @@ cat >$tmp/rules <<EOF
 ka:b:K1:::HEADER_ADD="X-Header: testing!!!"
 EOF
 
-MAILRULES=$tmp/rules sfecho <<EOF
+MAILRULES=$tmp/rules
+
+sfecho <<EOF
 MAIL FROM:<a>
 RCPT TO:<b>
 DATA
 .
 EOF
-MAILRULES=$tmp/rules sfecho <<EOF
+
+sfecho <<EOF
 MAIL FROM:<a>
 RCPT TO:<c>
 DATA
@@ -103,7 +103,9 @@ k:*:K1
 k*:*:K2
 EOF
 
-MAILRULES=$tmp/rules sfecho <<EOF
+MAILRULES=$tmp/rules
+
+sfecho <<EOF
 MAIL FROM:<>
 MAIL FROM:<foo>
 EOF
@@ -127,7 +129,9 @@ zx@y:four@four:ZZZ
 p*:five@five:PP
 EOF
 
-MAILRULES=$tmp/rules sfecho <<EOF
+MAILRULES=$tmp/rules
+
+sfecho <<EOF
 MAIL FROM:<nobody@nobody>
 RCPT TO:<one@one>
 RCPT TO:<two@two>
@@ -203,7 +207,9 @@ k!a@a:*:A
 k!!a@a:*:B
 EOF
 
-MAILRULES=$tmp/rules sfecho <<EOF
+MAILRULES=$tmp/rules
+
+sfecho <<EOF
 MAIL FROM:<a@a>
 MAIL FROM:<b@b>
 EOF
@@ -236,7 +242,9 @@ cat <<EOF | cdbmake $tmp/atlist.cdb $tmp/atlist.tmp
 
 EOF
 
-MAILRULES=$tmp/rules sfecho <<EOF
+MAILRULES=$tmp/rules
+
+sfecho <<EOF
 MAIL FROM:<a@a>
 MAIL FROM:<b@a>
 MAIL FROM:<a@b>
@@ -270,7 +278,9 @@ zfour@four:one@one:ZZ
 pfive@five:*:PP
 EOF
 
-MAILRULES=$tmp/rules sfecho <<EOF
+MAILRULES=$tmp/rules
+
+sfecho <<EOF
 MAIL FROM:<one@one>
 MAIL FROM:<two@two>
 MAIL FROM:<three@three>
@@ -331,14 +341,18 @@ END_OF_TEST_RESULTS
 ##### Test tests/smtpfront-maxnotimpl #####
 
 runtest() {
-MAXNOTIMPL=1 sfecho <<EOF
+MAXNOTIMPL=1
+
+sfecho <<EOF
 a
 b
 c
 d
 EOF
 
-MAXNOTIMPL=0 sfecho <<EOF
+MAXNOTIMPL=0
+
+sfecho <<EOF
 a
 b
 c
@@ -356,6 +370,62 @@ run_compare_test tests/smtpfront-maxnotimpl  <<END_OF_TEST_RESULTS
 END_OF_TEST_RESULTS
 
 
+##### Test tests/patterns-header #####
+
+runtest() {
+cat >$tmp/patterns <<EOF
+:header2:*field*
+EOF
+
+PATTERNS=$tmp/patterns
+
+sfecho <<EOF
+MAIL FROM:<>
+RCPT TO:<nobody>
+DATA
+header1: data
+header2: another field
+
+not
+
+also
+.
+EOF
+
+echo
+
+cat >$tmp/patterns <<EOF
+:not
+:also
+EOF
+
+sfecho <<EOF
+MAIL FROM:<>
+RCPT TO:<nobody>
+DATA
+header
+
+not
+
+also
+.
+EOF
+
+rm -f $tmp/patterns
+}
+run_compare_test tests/patterns-header  <<END_OF_TEST_RESULTS
+250 Sender=''.^M
+250 Recipient='nobody'.^M
+354 End your message with a period.^M
+554 This message contains prohibited content^M
+
+250 Sender=''.^M
+250 Recipient='nobody'.^M
+354 End your message with a period.^M
+250 Received 146 bytes.^M
+END_OF_TEST_RESULTS
+
+
 ##### Test tests/rules-both #####
 
 runtest() {
@@ -366,7 +436,9 @@ kb@a:a@b:K3
 kb@a:b@b:K4
 EOF
 
-MAILRULES=$tmp/rules sfecho <<EOF
+MAILRULES=$tmp/rules
+
+sfecho <<EOF
 MAIL FROM:<a@a>
 RCPT TO:<a@b>
 RCPT TO:<b@b>
@@ -405,7 +477,9 @@ cat >$tmp/atlist <<EOF
 b
 EOF
 
-MAILRULES=$tmp/rules sfecho <<EOF
+MAILRULES=$tmp/rules
+
+sfecho <<EOF
 MAIL FROM:<a@a>
 MAIL FROM:<b@a>
 MAIL FROM:<a@b>
@@ -435,7 +509,9 @@ cat >$tmp/rules <<EOF
 n*:*:Do not see this:::RELAYCLIENT=@rc
 EOF
 
-MAILRULES=$tmp/rules sfecho <<EOF
+MAILRULES=$tmp/rules
+
+sfecho <<EOF
 MAIL FROM:<a@a>
 RCPT TO:<b@b>
 EOF
@@ -458,7 +534,9 @@ cat >$tmp/patterns <<EOF
 =response 3
 EOF
 
-PATTERNS=$tmp/patterns sfecho <<EOF
+PATTERNS=$tmp/patterns
+
+sfecho <<EOF
 MAIL FROM:<>
 RCPT TO:<nobody>
 DATA
@@ -504,7 +582,9 @@ cat >$tmp/patterns <<EOF
 # comment
 EOF
 
-PATTERNS=$tmp/patterns sfecho <<EOF
+PATTERNS=$tmp/patterns
+
+sfecho <<EOF
 MAIL FROM:<>
 RCPT TO:<nobody>
 DATA
@@ -640,10 +720,75 @@ cur/1000000000.12345.here:1fsd
 END_OF_TEST_RESULTS
 
 
+##### Test tests/patterns-general #####
+
+runtest() {
+local subject="$1"
+cat >$tmp/patterns <<EOF
+/Subject: *word*
+EOF
+
+PATTERNS=$tmp/patterns
+
+sfecho <<EOF
+MAIL FROM:<>
+RCPT TO:<nobody>
+DATA
+Subject: $subject
+.
+EOF
+
+rm -f $tmp/patterns
+}
+run_compare_test tests/patterns-general 'xwordx' <<END_OF_TEST_RESULTS
+250 Sender=''.^M
+250 Recipient='nobody'.^M
+354 End your message with a period.^M
+554 This message contains prohibited content^M
+END_OF_TEST_RESULTS
+
+run_compare_test tests/patterns-general 'word at the beginning' <<END_OF_TEST_RESULTS
+250 Sender=''.^M
+250 Recipient='nobody'.^M
+354 End your message with a period.^M
+554 This message contains prohibited content^M
+END_OF_TEST_RESULTS
+
+run_compare_test tests/patterns-general 'last word' <<END_OF_TEST_RESULTS
+250 Sender=''.^M
+250 Recipient='nobody'.^M
+354 End your message with a period.^M
+554 This message contains prohibited content^M
+END_OF_TEST_RESULTS
+
+run_compare_test tests/patterns-general 'middle word middle' <<END_OF_TEST_RESULTS
+250 Sender=''.^M
+250 Recipient='nobody'.^M
+354 End your message with a period.^M
+554 This message contains prohibited content^M
+END_OF_TEST_RESULTS
+
+run_compare_test tests/patterns-general 'word' <<END_OF_TEST_RESULTS
+250 Sender=''.^M
+250 Recipient='nobody'.^M
+354 End your message with a period.^M
+554 This message contains prohibited content^M
+END_OF_TEST_RESULTS
+
+run_compare_test tests/patterns-general 'xord' <<END_OF_TEST_RESULTS
+250 Sender=''.^M
+250 Recipient='nobody'.^M
+354 End your message with a period.^M
+250 Received 142 bytes.^M
+END_OF_TEST_RESULTS
+
+
 ##### Test tests/smtpfront-looping-received #####
 
 runtest() {
-MAXHOPS=1 sfecho <<EOF
+MAXHOPS=1
+
+sfecho <<EOF
 MAIL FROM:<somebody>
 RCPT TO:<nobody@nowhere>
 DATA
@@ -653,7 +798,7 @@ EOF
 
 echo
 
-MAXHOPS=1 sfecho <<EOF
+sfecho <<EOF
 MAIL FROM:<somebody>
 RCPT TO:<nobody@nowhere>
 DATA
@@ -747,7 +892,9 @@ END_OF_TEST_RESULTS
 ##### Test tests/smtpfront-databytes #####
 
 runtest() {
-DATABYTES=123 sfecho <<EOF
+DATABYTES=123
+
+sfecho <<EOF
 EHLO hostname
 MAIL FROM:<somebody>
 MAIL FROM:<somebody> SIZE
@@ -895,7 +1042,9 @@ kone@one:two@two:A
 ktwo@two:*:B
 EOF
 
-MAILRULES=$tmp/rules sfecho <<EOF
+MAILRULES=$tmp/rules
+
+sfecho <<EOF
 MAIL FROM:<one@one>
 RCPT TO:<three@three>
 RCPT TO:<four@four>
@@ -972,6 +1121,77 @@ run_compare_test tests/smtpfront-auth-plain  <<END_OF_TEST_RESULTS
 END_OF_TEST_RESULTS
 
 
+##### Test tests/rules-databytes #####
+
+runtest() {
+cat >$tmp/rules <<EOF
+ksomebody:*::123
+EOF
+
+MAILRULES=$tmp/rules
+
+sfecho <<EOF
+EHLO hostname
+MAIL FROM:<somebody>
+MAIL FROM:<somebody> SIZE
+MAIL FROM:<somebody> SIZE=
+MAIL FROM:<somebody> SIZE=100
+MAIL FROM:<somebody> SIZE=123
+MAIL FROM:<somebody> SIZE=124
+RCPT TO:<nobody@nowhere>
+EOF
+}
+run_compare_test tests/rules-databytes  <<END_OF_TEST_RESULTS
+250-unknown^M
+250-SIZE 0^M
+250-8BITMIME^M
+250 PIPELINING^M
+250 OK^M
+250 OK^M
+250 OK^M
+250 OK^M
+250 OK^M
+552 The message would exceed the maximum message size.^M
+503 You must send MAIL FROM: first^M
+END_OF_TEST_RESULTS
+
+
+##### Test tests/rules-databytes2 #####
+
+runtest() {
+cat >$tmp/rules <<EOF
+k*:a::1
+k*:b::9999
+k*:c::
+EOF
+
+MAILRULES=$tmp/rules
+
+sfecho <<EOF
+EHLO hostname
+MAIL FROM:<somebody>
+RCPT TO:<a>
+RCPT TO:<b>
+RCPT TO:<c>
+DATA
+testing
+.
+EOF
+}
+run_compare_test tests/rules-databytes2  <<END_OF_TEST_RESULTS
+250-unknown^M
+250-SIZE 0^M
+250-8BITMIME^M
+250 PIPELINING^M
+250 Sender='somebody'.^M
+250 OK^M
+250 OK^M
+250 OK^M
+354 End your message with a period.^M
+552 Sorry, that message exceeds the maximum message length.^M
+END_OF_TEST_RESULTS
+
+
 ##### Test tests/rules-defaultmsg #####
 
 runtest() {
@@ -984,7 +1204,9 @@ z*:z
 k*:k
 EOF
 
-MAILRULES=$tmp/rules sfecho <<EOF
+MAILRULES=$tmp/rules
+
+sfecho <<EOF
 MAIL FROM:<d>
 MAIL FROM:<z>
 MAIL FROM:<k>
@@ -1033,7 +1255,10 @@ cat >$tmp/patterns <<EOF
 \after
 EOF
 
-PATTERNS=$tmp/patterns sfecho <<EOF
+PATTERNS=$tmp/patterns
+export PATTERNS
+
+sfecho <<EOF
 MAIL FROM:<>
 RCPT TO:<nobody>
 DATA
@@ -1049,7 +1274,7 @@ cat >$tmp/patterns <<EOF
 \before
 EOF
 
-PATTERNS=$tmp/patterns sfecho <<EOF
+sfecho <<EOF
 MAIL FROM:<>
 RCPT TO:<nobody>
 DATA
@@ -1081,7 +1306,10 @@ cat >$tmp/rules <<EOF
 ka:b:K1:::MAXHOPS=1
 EOF
 
-MAILRULES=$tmp/rules sfecho <<EOF
+MAILRULES=$tmp/rules
+export MAILRULES
+
+sfecho <<EOF
 MAIL FROM:<a>
 RCPT TO:<b>
 DATA
@@ -1115,7 +1343,10 @@ END_OF_TEST_RESULTS
 ##### Test tests/smtpfront-looping-delivered-to #####
 
 runtest() {
-MAXHOPS=1 sfecho <<EOF
+MAXHOPS=1
+export MAXHOPS
+
+sfecho <<EOF
 MAIL FROM:<somebody>
 RCPT TO:<nobody@nowhere>
 DATA
@@ -1125,7 +1356,7 @@ EOF
 
 echo
 
-MAXHOPS=1 sfecho <<EOF
+sfecho <<EOF
 MAIL FROM:<somebody>
 RCPT TO:<nobody@nowhere>
 DATA
@@ -1147,6 +1378,50 @@ run_compare_test tests/smtpfront-looping-delivered-to  <<END_OF_TEST_RESULTS
 END_OF_TEST_RESULTS
 
 
+##### Test tests/rules-rcptlist #####
+
+runtest() {
+cat >$tmp/rules <<EOF
+:r
+k*:[[$tmp/list]]:LIST
+k*:[[@$tmp/atlist]]:ATLIST
+d*:*:DD
+EOF
+
+cat >$tmp/list <<EOF
+a@a
+@c
+EOF
+
+cat >$tmp/atlist <<EOF
+b
+EOF
+
+MAILRULES=$tmp/rules $src/smtpfront-echo <<EOF 2>/dev/null | tail -n +2
+MAIL FROM:<nobody@nowhere>
+RCPT TO:<a@a>
+RCPT TO:<b@a>
+RCPT TO:<a@b>
+RCPT TO:<b@B>
+RCPT TO:<c@c>
+RCPT TO:<c@C>
+RCPT TO:<d@d>
+EOF
+
+rm -f $tmp/rules $tmp/list $tmp/atlist
+}
+run_compare_test tests/rules-rcptlist  <<END_OF_TEST_RESULTS
+250 Sender='nobody@nowhere'.^M
+250 LIST^M
+553 DD^M
+250 ATLIST^M
+250 ATLIST^M
+250 LIST^M
+250 LIST^M
+553 DD^M
+END_OF_TEST_RESULTS
+
+
 ##### Test tests/pop3front-auth-login #####
 
 runtest() {
@@ -1376,7 +1651,10 @@ ka@b:*:ONE\:TWO
 ka@c:*:ONE\\\\TWO:
 EOF
 
-MAILRULES=$tmp/rules sfecho <<EOF
+MAILRULES=$tmp/rules
+export MAILRULES
+
+sfecho <<EOF
 MAIL FROM:<a@a>
 MAIL FROM:<a@b>
 MAIL FROM:<a@c>
@@ -1402,7 +1680,10 @@ kc@*:*:K3
 k*:*:K4
 EOF
 
-MAILRULES=$tmp/rules sfecho <<EOF
+MAILRULES=$tmp/rules
+export MAILRULES
+
+sfecho <<EOF
 MAIL FROM:<a@a>
 MAIL FROM:<b@a>
 MAIL FROM:<c@b>
